/*
# Anti-Theft Intruder Alert System

This project uses an ESP32, ultrasonic sensor, and IR blaster to detect motion within a defined distance.
If movement is detected, it turns on a Samsung TV (as a scare tactic) and sends an alert to a Make.com webhook.
The system can be remotely enabled or disabled using the Blynk app.

## Hardware Requirements
- ESP32 development board  
- Ultrasonic distance sensor (HC-SR04 or similar)  
- IR LED for sending Samsung TV remote codes  
- Power supply for ESP32  
- Smartphone with Blynk app installed  

## Pin Connections
- Ultrasonic Trigger Pin: GPIO 16  
- Ultrasonic Echo Pin: GPIO 17  
- IR Send Pin: GPIO 4  

## Features
- Remotely enable or disable the system via Blynk (V3)  
- Monitors distance in front of the sensor  
- If an object is detected closer than 80 cm:
  - Turns on Samsung TV using IR remote signal  
  - Sends an alert with distance info via Make.com webhook  
- Prevents repeated notifications until object is cleared  
- Serial output for real-time monitoring and debugging  

## Video
- https://youtu.be/qRPGj36vkL0?t=166

## Credits
- Theodore_Dai_Maman #211541594  
- Omer_Dan #322952466  
- Amit_Kaminsky #207487661  

## Setup Instructions
1. Install required libraries in Arduino IDE:  
   - Blynk  
   - WiFi  
   - HTTPClient  
   - IRremote  

2. Update the following in the code:  
   - WiFi credentials (`ssid` and `password`)  
   - `BLYNK_AUTH_TOKEN` with your Blynk auth token  
   - `makeWebhookUrl` with your Make.com webhook URL  

3. Upload the code to your ESP32  

## Notes
- Distance threshold can be adjusted via `DISTANCE_THRESHOLD`  
- Works best in environments without too much background movement  
- Make sure the IR LED is facing the TV with a clear line of sight  
- The system checks for movement every 5 seconds when enabled  
*/

#define BLYNK_TEMPLATE_ID           "Enter Template ID"
#define BLYNK_TEMPLATE_NAME         "Enter Template Name"
#define BLYNK_AUTH_TOKEN            "Enter Auth Token"
#define BLYNK_PRINT Serial

#include <WiFi.h>
#include <BlynkSimpleEsp32.h>
#include <HTTPClient.h>
#include <IRremote.hpp>

// WiFi credentials
const char* ssid = "Enter SSID";
const char* password = "Enter Password";

// Pin definitions
#define TRIGGER_PIN  16
#define ECHO_PIN     17
#define MAX_DISTANCE 150
#define IR_SEND_PIN 4

// Distance threshold (in cm)
const int DISTANCE_THRESHOLD = 80;

// Make.com webhook URL for notifications
const char* makeWebhookUrl = "Enter Webhook URL";

// Variables
bool alarmEnabled = false;
bool notificationSent = false;
unsigned long lastCheckTime = 0;
const unsigned long CHECK_INTERVAL = 5000;  // Check every 5 seconds

// Samsung TV IR codes
uint8_t sAddress = 0x7;
uint8_t sCommand = 0xE6;
uint8_t sRepeats = 0;

// This function will be called every time V3 changes
BLYNK_WRITE(V3) {
  Serial.println("Starting to read alarm enabled");
  alarmEnabled = param.asInt();
  Serial.println("Alarm enabled read complete, alarmEnabled = " + String(alarmEnabled));
}

void setup() {
  Serial.begin(115200);
  Serial.println("\nStarting Anti-Theft System");
  
  // Initialize IR sender
  IrSender.begin(IR_SEND_PIN);
  Serial.println("IR sender initialized");
  
  // Connect to WiFi
  Serial.print("Connecting to WiFi");
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nWiFi Connected!");

  // Connect to Blynk
  Serial.print("Connecting to Blynk...");
  Blynk.begin(BLYNK_AUTH_TOKEN, ssid, "");
  Serial.println("Connected!");
}

void loop() {
  Blynk.run();

  if (alarmEnabled) {
    checkDistance();
    delay(5000);  // Same delay as in SetAlarm
  }
}

void checkDistance() {
  int distance = ultrasonic_measure(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);
  
  Serial.print("Distance: ");
  Serial.print(distance);
  Serial.print(" cm | Alarm Enabled: ");
  Serial.println(alarmEnabled ? "YES" : "NO");
  
  // Check conditions and trigger actions if needed
  if (distance < DISTANCE_THRESHOLD && distance != 0) {
    if (!notificationSent) {
      // Turn on TV
      Serial.println("Turning on TV");
      IrSender.sendSamsung(sAddress, sCommand, sRepeats);
      Blynk.virtualWrite(V5, 1);  // Send value 1 to V5
      
      // Send notification
      sendMakeNotification(distance);
      notificationSent = true;  // Prevent spam notifications
    }
  } else {
    notificationSent = false;  // Reset notification flag
  }
}

int ultrasonic_measure(int trigPin, int echoPin, int max_distance) {
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(5);
  digitalWrite(trigPin, LOW);
  int duration = pulseIn(echoPin, HIGH, max_distance * 59);
  return duration / 59;
}

void sendMakeNotification(int distance) {
  if (WiFi.status() == WL_CONNECTED) {
    HTTPClient http;
    http.begin(makeWebhookUrl);
    http.addHeader("Content-Type", "application/json");
    
    // Create JSON payload
    String payload = "{\"alert\":\"Intruder detected! Distance: " + String(distance) + " cm\"}";
    
    // Send POST request
    int httpResponseCode = http.POST(payload);
    
    if (httpResponseCode > 0) {
      Serial.print("Make.com notification sent. Response code: ");
      Serial.println(httpResponseCode);
    } else {
      Serial.print("Error sending notification. Error code: ");
      Serial.println(httpResponseCode);
    }
    
    http.end();
  }
} 